import pytest
import socket


def _check_network():
    sock = None
    try:
        sock = socket.create_connection(('www.google.com', 80))
        sock.close()
        return True
    except Exception:
        if sock:
            sock.close()
        return False


requires_network = pytest.mark.skipif(not _check_network(), reason='This test requires a network connection.')



try:
    from unittest import mock
except (ImportError, AttributeError):
    import mock

import pytest
import socket
import rfc6555
from .test_utils import requires_network


class _BasicCreateConnectionTests(object):
    @requires_network
    def test_create_connection_google(self):
        sock = rfc6555.create_connection(('www.google.com', 80))

    @pytest.mark.parametrize('timeout', [None, 5.0])
    def test_create_connection_has_proper_timeout(self, timeout):
        sock = rfc6555.create_connection(('www.google.com', 80), timeout=timeout)

        assert sock.gettimeout() == timeout

    def test_create_connection_with_source_address_calls_bind(self):
        sock = mock.Mock()
        with mock.patch('socket.socket') as fake_socket:
            fake_socket.return_value = sock

            sock.getsockopt.return_value = 0
            sock.connect_ex.return_value = 0
            sock.gettimeout.return_value = None

            try:
                rfc6555.create_connection(('::1', 0), source_address=('::1', 123))
            except Exception:
                pass

            sock.bind.assert_called_with(('::1', 123))

    def test_getaddr_info_empty_list(self):
        with mock.patch('socket.getaddrinfo') as fake_getaddrinfo:
            fake_getaddrinfo.return_value = []

            with pytest.raises(socket.error):
                rfc6555.create_connection(('::1', 0))

    @requires_network
    def test_create_connection_cached_value(self):
        sock = rfc6555.create_connection(('www.google.com', 80))
        sock2 = rfc6555.create_connection(('www.google.com', 80))


class TestCreateConnectionTestRFC6555Default(_BasicCreateConnectionTests):
    pass


class TestCreateConnectionTestRFC6555Enabled(_BasicCreateConnectionTests):
    def setup_method(self, test_method):
        rfc6555.RFC6555_ENABLED = True


class TestCreateConnectionTestRFC6555Disabled(_BasicCreateConnectionTests):
    def setup_method(self, test_method):
        rfc6555.RFC6555_ENABLED = False



import os
import re
from setuptools import setup

# Get the version (borrowed from SQLAlchemy)
base_path = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(base_path, 'rfc6555.py')) as f:
    VERSION = re.compile(r'.*__version__ = \'(.*?)\'', re.S).match(f.read()).group(1)

with open(os.path.join(base_path, 'README.rst')) as f:
    long_description = f.read()

with open(os.path.join(base_path, 'CHANGES.rst')) as f:
    changes = f.read()

if __name__ == '__main__':
    setup(name='rfc6555',
          description='Python implementation of the Happy Eyeballs Algorithm described in RFC 6555.',
          long_description=long_description + '\n\n' + changes,
          license='Apache-2.0',
          url='https://www.github.com/SethMichaelLarson/rfc6555',
          version=VERSION,
          author='Seth Michael Larson',
          author_email='sethmichaellarson@protonmail.com',
          maintainer='Seth Michael Larson',
          maintainer_email='sethmichaellarson@protonmail.com',
          install_requires=[
              'selectors2;python_version<"3.4"'
          ],
          py_modules=['rfc6555'],
          zip_safe=False,
          classifiers=['Intended Audience :: Developers',
                       'License :: OSI Approved :: Apache Software License',
                       'Natural Language :: English',
                       'Operating System :: OS Independent',
                       'Programming Language :: Python :: 2',
                       'Programming Language :: Python :: 2.6',
                       'Programming Language :: Python :: 2.7',
                       'Programming Language :: Python :: 3',
                       'Programming Language :: Python :: 3.3',
                       'Programming Language :: Python :: 3.4',
                       'Programming Language :: Python :: 3.5',
                       'Programming Language :: Python :: 3.6',
                       'Topic :: Internet',
                       'Topic :: System :: Networking'])



import socket
import rfc6555

try:
    from unittest import mock
except (ImportError, AttributeError):
    import mock


def test_ipv6_available():
    assert rfc6555._detect_ipv6()


def test_ipv6_not_available_socket_has_ipv6_false():
    old_has_ipv6 = socket.has_ipv6
    socket.has_ipv6 = False
    assert not rfc6555._detect_ipv6()
    socket.has_ipv6 = old_has_ipv6


def test_ipv6_not_available_socket_exception_on_init():
    with mock.patch('socket.socket') as fake_socket:
        fake_socket.side_effect = OSError

        assert not rfc6555._detect_ipv6()


def test_ipv6_not_available_socket_exception_on_bind():
    sock = mock.Mock()
    with mock.patch('socket.socket') as fake_socket:
        fake_socket.return_value = sock
        sock.bind.side_effect = OSError

        assert not rfc6555._detect_ipv6()


def test_ipv6_not_available_socket_AF_INET6_not_defined():
    old_AF_INET6 = socket.AF_INET6
    try:
        delattr(socket, 'AF_INET6')
        assert not rfc6555._detect_ipv6()
    finally:
        socket.AF_INET6 = old_AF_INET6
